function [Prob]=TestingFractionDiffRDA(Y,X,W,NumberBootSamples);

% TestingFractionDiffRDA: compares fractions of variation in RDA
% based on two data matrices X and W.  
% Tests of significance are performed by bootstrap. 
% input:
%   Y = data matrix (obs x vars) of response variables
%   X and W = data matrices (obs x vars) of independent variables
%   NumberBootSamples = Number of bootstrap samples for the test (e.g., 99, 999, 9999) - the observed statistic
%   is included in the bootstrapped distribution; 
% output:
%   Prob = probability of that fractions [a] and [c] come from the same statistical population of explained fractions of variation and that 
%          they only differ by sampling variation.
%
% author: Pedro Peres-Neto, May 2005 (pedro.peres-neto@uregina.ca)
      
m=size(X,2);
q=size(W,2);
n=size(Y,1);
t=size(Y,2);

% centering data
Y = Y - ones(n,1)*mean(Y);
X = X - ones(n,1)*mean(X);
W = W - ones(n,1)*mean(W);

TotalMS=trace(Y'*Y)/(n-1);
TotalMSMaxLik=trace(Y'*Y)/n; % note that MS is divided by n and not by n-1

PX=X*inv(X'*X)*X';
predXObs=PX*Y;
residualSS=trace((Y-predXObs)'*(Y-predXObs));
abAdjObs=1-(residualSS/(n-m-1))/TotalMS;
residualsX=sqrt(n/(n-m))*(Y-predXObs); % residuals are standardized

PW=W*inv(W'*W)*W';
predWObs=PW*Y;
residualSS=trace((Y-predWObs)'*(Y-predWObs));
bcAdjObs=1-(residualSS/(n-q-1))/TotalMS;
residualsW=sqrt(n/(n-q))*(Y-predWObs); % residuals are standardized

for i=1:NumberBootSamples-1
    BootIdx=ceil(rand(n,1)*n);
    BootSampleResX=residualsX(BootIdx,:);
    BootSampleResW=residualsW(BootIdx,:);

    % ab
    YBoot=predXObs+BootSampleResX; 
    YBoot = YBoot - ones(n,1)*mean(YBoot);
    TotalSS=trace(YBoot'*YBoot);
    predX=PX*YBoot;
    residualSS=trace((YBoot-predX)'*(YBoot-predX));
    ABRsqAdj(i,1)=1-(residualSS/(n-m-1))/TotalMSMaxLik;

    %bc
    YBoot=predWObs+BootSampleResW; 
    YBoot = YBoot - ones(n,1)*mean(YBoot);
    TotalSS=trace(YBoot'*YBoot);
    predW=PW*YBoot;
    residualSS=trace((YBoot-predW)'*(YBoot-predW));
    BCRsqAdj(i,1)=1-(residualSS/(n-q-1))/TotalMSMaxLik;

    RsqDiffABBC(i,1)=ABRsqAdj(i,1)-BCRsqAdj(i,1);
end;
RsqDiffABBC(NumberBootSamples,1)=abAdjObs-bcAdjObs; % includes observed value
RsqDiffABBC=sort(RsqDiffABBC);

if median(RsqDiffABBC) <=0 
   Prob=size(find(RsqDiffABBC>=0),1)/NumberBootSamples; 
else
   Prob=size(find(RsqDiffABBC<=0),1)/NumberBootSamples;  
end;
Prob=Prob*2;






   


