# R script for null model 
# the colnames of input table must be set as species name whereas rownames should be sites name


null_model <- function(otutable,reps=10000){
  library(vegan)
  
  # Fristly, we calculate the sample counts and gamma diversity that we define as 'species pool'
  site_counts <- nrow(otutable)
  species_pool <- ncol(otutable)
  
  # here we make the absolute abundence data into a presence-absence matrix
  # A value of 1 represents presence whereas 0 suggests that one OTU or species not exist in a given site or sample
  otutable <- ceiling(otutable/max(otutable))
  
  # calculate the proportion of sites each OTUs or species occupies
  # this vector will be used in null model later
  probs <- apply(otutable, 2, FUN = sum)

  # determine how many unique species richness values are in the dataset
  # this is used to limit the number of null communities that have to be calculated, that is how many species we should sample in species pool
  alpha_levels<-sort(unique(apply(otutable, MARGIN=1, FUN=sum)))
  
  # make null model:
  # alpha_table is used as a lookup to help identify which null distribution to use for the tests later.  
  # It contains one row for each combination of alpha richness levels. 
  alpha_table<-data.frame(c(NA), c(NA))
  names(alpha_table)<-c("smaller_alpha", "bigger_alpha")
  col_count<-1
  
  # a list be created to save null distribution
  null_array<-list()
  
  # looping over each combination of alpha levels:
  for(a1 in 1:length(alpha_levels)){
    for(a2 in a1:length(alpha_levels)){
      
      # build a null distribution of the number of shared species for a pair of alpha values:
      null_shared_spp <- NULL
      for(i in 1:reps){
        
        # two empty null communities of size identical to species_pool:
        com1 <- rep(0,species_pool)
        com2 <- rep(0,species_pool)
        
        # add alpha1 number of species to com1, weighting by species occurrence frequencies:
        com1[sample(1:species_pool, alpha_levels[a1], replace = FALSE, prob = probs)] <- 1
        
        # same for com2:
        com2[sample(1:species_pool, alpha_levels[a2], replace = FALSE, prob = probs)] <- 1
        
        # calculate species are shared in common, that is null distribution (SSexp)
        null_shared_spp[i] <- sum((com1+com2)>1)
        
      }
      
      # save null distribution (SSexp) into a list
      null_array[[col_count]] <- null_shared_spp
      
      # record values for alpha 1 and 2 in the alpha_table to help find the correct null distribution later
      alpha_table[col_count, which(names(alpha_table) == "smaller_alpha")] <- alpha_levels[a1]
      alpha_table[col_count, which(names(alpha_table) == "bigger_alpha")] <- alpha_levels[a2]
      col_count <- col_count+1
      
    }
  }
  # create a new column with both alpha levels to match on:
  alpha_table$matching <- paste(alpha_table[,1], alpha_table[,2], sep="_")
  ## null distribution calculation end
  
  
  ## calculate the observed share species
  
  # Three null matrix will be created to save ses, observed jaccard index and expected jaccard index
  sd_cal <- matrix(data=NA, nrow=site_counts, ncol=site_counts, dimnames=list(row.names(otutable), row.names(otutable)))
  Jac_obs <- matrix(data=NA, nrow=site_counts, ncol=site_counts, dimnames=list(row.names(otutable), row.names(otutable)))
  Jac_exp <- matrix(data=NA, nrow=site_counts, ncol=site_counts, dimnames=list(row.names(otutable), row.names(otutable)))
  
  
  for(i in 1:site_counts){
    for(j in 1:site_counts){
      # calculate the observed share species or OTUs
      n_shared_obs <- sum((otutable[i,] + otutable[j,]) > 1)
      
      # calculate the observed richness of each site
      obs_a1 <- sum(otutable[i,])
      obs_a2 <- sum(otutable[j,])
      
      # place these alphas into an object to match against alpha_table (sort so smaller alpha is first)
      obs_a_pair <- sort(c(obs_a1, obs_a2))
      
      ##match against the alpha table- row index identifies which element of the null array contains the correct null distribution for the observed combination of alpha values:
      null_index <- which(alpha_table$matching == paste(obs_a_pair[1], obs_a_pair[2], sep="_"))
      
## //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# All of the code above was provided by Chase et al (2011)
## On the basis of Chase, we calculated the relative importance of stochasticity and deterministic factor in community assambly
##//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

      # first of all, we calculate the observed share species in common in two given sites
      null_distribution <- null_array[[null_index]] # return a vector
      
      # a null vector will be created
      # calculate jaccard similarity index and standard effect size
      Jexp <- NULL # a null vector of expected jaccard's index 
      for (n in seq(from = 1, to = length(null_distribution),by = 1)){
	    SSexp <- null_distribution[n] # return a expected shared otu number
        Je <- (SSexp)/(obs_a1 + obs_a2 - SSexp) #calculated expected jaccard's dissimilarity index 
		Je <- 1-Je #jaccard's dissimilarity index
        Jexp <- c(Jexp, Je)
      }
      mean_Jexp <- mean(Jexp) #the mean expected jaccard's index of 1000 random sampling
      sd_Jexp <- sd(Jexp) #The expected jaccard's index sd of 1000 random sampling
      Jobs <- n_shared_obs/(obs_a1+obs_a2-n_shared_obs) #the observed jaccard's index of two community 
      Jobs <- 1- Jobs#jaccard's dissimilarity index
	  sd_cal[i,j] <- sd_Jexp # standard effect size (SES) matrix
      Jac_obs[i,j] <- Jobs # observed jaccard's index matrix
      Jac_exp[i,j] <- mean_Jexp # expected jaccard's index matrix
    }
  }
  result <- list()
  result$expected_sd <- sd_cal
  result$expected_jaccard <- Jac_exp
  result$observed_jaccard <- Jac_obs
  return(result)
}

